var fs = require("fs");
var path = require("path");
var babel = require("babel-core");

var sep = path.sep;

function asyncList (list, callback){
	var count = list.length,
		results = [];
	if(list && count > 0){
		list.forEach(function(item, index) {
			item(function(result) {
				results[index] = result;
				count --;
				if(count === 0){
					callback.apply(null, results);
				}
			});
		});
	}else{
		callback();
	}
};

var ignore = [];
function readFiles (dir, ext, isDeep, callback){
	var files = [];

	dir = dir.replace(/(\/|\\)$/, "");

	fs.readdir(dir, function(err, _files){
		if(err){
			throw err;
		}

		var dirs = [],
			readStatList = [];
		_files.forEach(function(filename){
			if(ignore.indexOf(filename) !== -1){
				return;
			}

			var file = dir + sep + filename;

			readStatList.push(function(callback){
				fs.stat(file, function(err, stats){
					if(err){
						throw err;
					}

					var _ext = path.extname(filename);

					if(stats.isDirectory()){
						dirs.push(function(callback){
							readFiles(file, ext, isDeep, function(_files){
								files = files.concat(_files);
								callback();
							});
						});
						if(ext === "/"){
							files.push(file);
						}
					}else{
						if(ext){
							if(_ext === ext){
								files.push(file);
							}
						}else{
							files.push(file);
						}
					}
					callback();
				});
			});
		});

		asyncList(readStatList, function(){
			if(isDeep && dirs.length){
				asyncList(dirs, function(){
					callback(files);
				});
			}else{
				callback(files);
			}
		});
	});
};

function mkdirs(dirpath, callback) {
    if(callback){
        fs.exists(dirpath, function(exists) {
            if(exists) {
                    callback(dirpath);
            } else {
                    //尝试创建父目录，然后再创建当前目录
                    mkdirs(path.dirname(dirpath), function(){
                            fs.mkdir(dirpath, callback);
                    });
            }
        });
    }else{
        if(fs.existsSync(dirpath)){
            return true;
        }else{
            if(mkdirs(path.dirname(dirpath))){
                fs.mkdirSync(dirpath);
                return true;
            }
        }
    }
};

function write(file, content){
	mkdirs(path.dirname(file), function(){
		fs.writeFile(file, content, function(err){
			if(err){
				throw err;
			}
		});
	});
}

var cssTpl = fs.readFileSync(path.join(__dirname, "css.tpl"), {
		encoding: "utf8"
	});

readFiles(path.join(__dirname, "src"), null, true, function(files){
	files.forEach(function(file){
		var distFile = file.replace("/src/", "/dist/");

		// var _file = file.split("/src/");
		// _file.shift();
		// _file = _file.join("/src/");
		// var distFile = "/Users/lifan/work-elong/h5-eroom-enjoy/MinSu/web/weixin/node_modules/enjoy-weixin-support/dist/" + _file;

		var ext = path.extname(file);

		if(ext === ".js"){
			babel.transformFile(file, {
				presets: ['es2015', "stage-0"],
				plugins: ["transform-react-jsx", "transform-decorators-legacy"],
			}, function(err, result){
				if(err){
					throw err;
				}

				result = babel.transformFromAst(result.ast, result.code, {
					plugins: [function () {
						return {
							visitor: {
								CallExpression: {
									enter(path){
										var node = path.node;
										if(node.callee.type === "Identifier" && node.callee.name === "require" && node.arguments[0].type === "StringLiteral"){
											node.arguments[0].value = node.arguments[0].value.replace(/\.(css)$/, "-$1");
										}else if(node.callee.type === "MemberExpression" &&
											// node.callee.object.type === "Identifier" &&
											// node.callee.object.name === "React" &&
											node.callee.property.type === "Identifier" &&
											node.callee.property.name === "createElement" &&
											node.arguments[1] &&
											node.arguments[1].type === "ObjectExpression"){
											let properties = node.arguments[1].properties;
											properties.forEach(property => {
												var key = property.key;
												switch(key.value){
													case "class":
														key.value = "className";
														break;
													case "for":
														key.value = "htmlFor";
														break;
												}
											});
										}
									}
								}
							}
						};
					}]
				});

				write(distFile, result.code);
			});
		}else if(ext === ".css"){
			fs.readFile(file, {
				encoding: "utf8"
			}, function(err, result){
				if(err){
					throw err;
				}

				write(distFile.replace(/\.css$/, "-css.js"), cssTpl.replace("{content}", result.replace(/[\n\r\t]+/g, " ")));
			});
		}else{
			fs.readFile(file, {
				encoding: "utf8"
			}, function(err, result){
				if(err){
					throw err;
				}

				write(distFile, result);
			});
		}
	});
});